# Example: look at track resolution using single particles
# hs-get pgun_elec_eta4_pt30gev%rfull052 data  2 100  
# S.Chekanov (ANL)


from org.lcsim.lcio import LCIOReader
from hep.io.sio import SIOReader
from hep.lcio.implementation.sio import SIOLCReader
from hep.lcio.implementation.io import LCFactory
from hep.lcio.event import * 
from hep.lcio.io import *
from jhplot import *  # import graphics
from hephysics.particle import LParticle
from java.lang import System
from java.awt import Color
import math


# pT bins (min, max)
ptbins=[[0,5],[5,10],[15,20],[20,25],[25,30]]

# fit ranges 
xmin=0.8
xmax=1.2

# make histograms for ratio and pT bins
h_re=[] # histogram with ratio 
h_pt=[] # histogram with pT 
print "Nr of bins=",len(ptbins)
for j in range(len(ptbins)):
        pbin=ptbins[j]
        lab=str(pbin[0])+"-"+str(pbin[1])
        h_re.append(H1D("ratio: "+lab,200,xmin,xmax))
        h_pt.append(H1D("bin: "+lab,100,pbin[0],pbin[1]))

# make list of files..
import glob
files=glob.glob("data/*.slcio")
factory = LCFactory.getInstance()

nEvent=0

for f in files:
  print "Open file=",f
  reader = factory.createLCReader()
  reader.open(f) 
  while(1):
    evt=reader.readNextEvent()
    if (evt == None): break
    nEvent=nEvent+1
    # print " file event: ",evt.getEventNumber(), " run=",evt.getRunNumber()
    if (nEvent%100==0): print "# Event: ",nEvent
    col = evt.getCollection("MCParticle")
    nMc=col.getNumberOfElements()
    particles=[]
    pp=0
    nn=0
    for i in range(nMc): # loop over all particles 
      par=col.getElementAt(i)
      if(par.getGeneratorStatus() == 1 and par.getCharge() !=0): 
              vertex = par.getVertex();
              pdg=par.getPDG()
              momentum = par.getMomentum() 
              ee=par.getEnergy()
              mass=par.getMass()
              px=momentum[0]
              py=momentum[1]
              pz=momentum[2]
              charge=par.getCharge()
              p=LParticle(px,py,pz,ee,mass) 
              p.setCharge(charge)
              particles.append(p)
              #print nn, "pdg=",pdg, " ",px,py,pz, " charge=",charge 
              nn=nn+1

    pfo=[]
    col = evt.getCollection("PandoraPFOCollection")
    nPFA = col.getNumberOfElements()
    for i in range(nPFA):
       # http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/lcio/SIOReconstructedParticle.html
       # http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/event/ReconstructedParticle.html
       pa=col.getElementAt(i)
       if len(pa.getTracks())>0: # tracking 
          charge=pa.getCharge()
          p4=pa.getMomentum()
          ee=pa.getEnergy()
          typep=pa.getType() # type
          p=LParticle(p4[0],p4[1],p4[2],ee)
          p.setCharge(charge)
          pfo.append(p)

    # match reco with truth  
    for i1 in range(len(pfo)):
          p1=pfo[i1]
          pt=p1.perp()
          phi=p1.phi()
          rap=p1.rapidity()
          for i2 in range(len(particles)):
            p2=particles[i2]
            t_pt=p2.perp()
            t_phi=p2.phi()
            t_rap=p2.rapidity()
            dphi=(t_phi-phi)
            if (dphi>math.pi): dphi=2*math.pi-dphi
            deta=(t_rap-rap)
            dd=math.sqrt(dphi*dphi+deta*deta)
            if (dd<0.1):
                        rr=pt/t_pt
                        for j in range(len(ptbins)):
                              pbin=ptbins[j]
                              if(t_pt>pbin[0] and t_pt<pbin[1]):
                                       h_re[j].fill(rr)
                                       h_pt[j].fill(t_pt)  


    del col
    del evt 
  reader.close() # close the file
  del reader
  System.gc() # if you low on memory, manual JVM cleanup for every 5th file 

pres=P1D("track resolution")
# fit histograms with a Gaussian
for i in range(len(ptbins)):
    pbin=ptbins[i]
    lab=str(pbin[0])+"-"+str(pbin[1])
    h=h_re[i]
    p=h_pt[i]
    f=HFitter("chi2")   # build fitter 
    f.setFunc("g")      # gaussian + constant  
    func=f.getFunc()
    #print func.parameterNames()
    f.setPar('mean',h.mean()); f.setPar('amplitude',h.maxBinHeight()), f.setPar('sigma',h.rms()) 
    f.setRange(xmin,xmax) # fit range 
    f.fit(h)
    fresult=f.getFittedFunc() # return final function 
    res=f.getResult()
    fPars  = res.fittedParameters()
    fPars     = res.fittedParameters()
    fParErrs  = res.errors()
    fParNames = res.fittedParameterNames()
    print "Fit results:"
    for i in range(fresult.numberOfParameters()):
       print "par=",i,"=", fParNames[i]+" : "+str(fPars[i])+" +- "+str(fParErrs[i])
    p2=P1D(h)   # convert histogram to P1D array
    # c1.draw(p2)
    # make F1D function from IFunction in the same range as histogram
    f2 = F1D("Gauss fit",fresult,xmin,xmax)
    f2.setColor(Color.blue)
    f2.setPenWidth(2);
    # c1.draw(f2) # draw fit line  
    q1="mean=%4.3f &plusmn; %4.3f GeV"%(fPars[1],fParErrs[1])
    print "pT=",p.mean()," GeV ", q1

    c1=HPlot("PT="+lab,600,400)
    c1.visible()
    c1.setMarginLeft(80)
    c1.setNameX("pT(jet, PFA) / pT(jet, truth)")
    c1.setNameY("Events:"+lab)
    c1.setAutoRange()
    c1.setRangeX(xmin,xmax)
    c1.draw(p2)
    c1.draw(f2)
    # x, y, err_x, err_x, err_y, err_y
    pres.add(p.mean(),fPars[2],p.rms(),p.rms(),fParErrs[2],fParErrs[2])

# final plot
c1=HPlot("resoluion",600,400)
c1.visible()
c1.setNameX("pT [GeV]")
c1.setNameY("&delta; pT / pT")
c1.setAutoRange()
c1.setMarginLeft(100)
c1.setRangeX(0,40)
c1.draw(pres)




